// Copyright 2021-present 650 Industries. All rights reserved.
// Copyright 2022 Hirbod Mirjavadi. All rights reserved.

import CoreGraphics
import Photos
import UIKit
import ExpoModulesCore
import NSFWDetector

public class RNNSFWDetectorModule: Module {
  typealias LoadImageCallback = (Result<UIImage, Error>) -> Void

  guard #available(iOS 12.0, *), let detector = NSFWDetector.shared else {
      return
  }

  public func definition() -> ModuleDefinition {
    Name("RNNSFWDetector")

    AsyncFunction("detectAsync", manipulateImage)
      .runOnQueue(.main)
  }

  internal func detectNSFW(url: URL, threshold: ConfidenceThreshold, promise: Promise) {
    loadImage(atUrl: url) { result in
      switch result {
      case .failure(let error):
        return promise.reject(error)
      case .success(let image):
        do {

          detector.check(image: image, completion: { result in
              switch result {
              case let .success(nsfwConfidence: confidence):
                  if confidence > 0.9 {
                    promise.resolve([
                      "isNSFW": true,
                      "confidence": confidence
                    ])
                  } else {
                    promise.resolve([
                      "isNSFW": false,
                      "confidence": confidence
                    ])
                  }
              default:
                promise.reject(error)
              }
          })

        } catch {
          promise.reject(error)
        }
      }
    }
  }

  /**
   Loads the image from given URL.
   */
  internal func loadImage(atUrl url: URL, callback: @escaping LoadImageCallback) {
    if url.scheme == "data" {
      guard let data = try? Data(contentsOf: url), let image = UIImage(data: data) else {
        return callback(.failure(CorruptedImageDataException()))
      }
      return callback(.success(image))
    }
    if url.scheme == "assets-library" {
      // TODO: ALAsset URLs are deprecated as of iOS 11, we should migrate to `ph://` soon.
      return loadImageFromPhotoLibrary(url: url, callback: callback)
    }

    guard let imageLoader = self.appContext?.imageLoader else {
      return callback(.failure(ImageLoaderNotFoundException()))
    }
    guard let fileSystem = self.appContext?.fileSystem else {
      return callback(.failure(FileSystemNotFoundException()))
    }
    guard fileSystem.permissions(forURI: url).contains(.read) else {
      return callback(.failure(FileSystemReadPermissionException(url.absoluteString)))
    }

    imageLoader.loadImage(for: url) { error, image in
      guard let image = image, error == nil else {
        return callback(.failure(ImageLoadingFailedException(error.debugDescription)))
      }
      callback(.success(image))
    }
  }

  /**
   Loads the image from user's photo library.
   */
  internal func loadImageFromPhotoLibrary(url: URL, callback: @escaping LoadImageCallback) {
    guard let asset = PHAsset.fetchAssets(withALAssetURLs: [url], options: nil).firstObject else {
      return callback(.failure(ImageNotFoundException()))
    }
    let size = CGSize(width: asset.pixelWidth, height: asset.pixelHeight)
    let options = PHImageRequestOptions()

    options.resizeMode = .exact
    options.isNetworkAccessAllowed = true
    options.isSynchronous = true
    options.deliveryMode = .highQualityFormat

    PHImageManager.default().requestImage(for: asset, targetSize: size, contentMode: .aspectFit, options: options) { image, _ in
      guard let image = image else {
        return callback(.failure(ImageNotFoundException()))
      }
      return callback(.success(image))
    }
  }

}
